#include <msp430g2553.h>
#include <stdint.h>

#include "hmi.h"
#include "engine.h"
#include "debug.h"


class SwitchPort {
public:
	SwitchPort(uint16_t bit, uint16_t inRegister) : m_bit(bit), m_state(0), m_cnt(0), m_inRegister(inRegister) {
	};
	bool get() {
		bool result = false;
		bool inValue = false;
		switch (m_inRegister) {
		case 1:
			inValue = P1IN & m_bit;
			break;
		case 2:
			inValue = P2IN & m_bit;
			break;
		}
		switch (m_state) {
		case 0:

			if (inValue) {
				m_cnt = 0;
			} else {
				m_cnt++;
			}
			if (m_cnt > 100) {
				m_state = 1;
				m_cnt = 0;
			}
			break;
		case 1:
			result = true;
			m_state = 2;
			break;
		case 2:
			if (inValue) {
				m_cnt++;
			} else {
				m_cnt = 0;
			}
			if (m_cnt > 20) {
				m_state = 0;
				m_cnt = 0;
			}
			break;
		default:
			m_state = 0;
			break;
		}
		return result;
	};
private:
	uint16_t m_bit;
	uint8_t m_state;
	uint16_t m_cnt;
	uint8_t m_inRegister;
};

class SwitchPort2 : public SwitchPort {
public:
	SwitchPort2(uint16_t bit) : SwitchPort(bit, 2) {
		P2REN |= bit;
		P2DIR &= ~bit;
		P2SEL &= ~bit;
	};
};
class SwitchPort1 : public SwitchPort {
public:
	SwitchPort1(uint16_t bit) : SwitchPort(bit, 1) {
		P1REN |= bit;
		P1DIR &= ~bit;
		P1SEL &= ~bit;
	};
};


SwitchPort intensitySwitch = SwitchPort2(BIT1);
SwitchPort onTimeSwitch = SwitchPort2(BIT2);
SwitchPort offTimeSwitch = SwitchPort2(BIT3);
//SwitchPort modeSwitch = SwitchPort2(BIT4);

int16_t intensity = 0;
uint16_t maxIntensity = 0;
uint16_t intensityStep = 0;
const uint16_t intensityStepCnt = 16;


void hmiInit() {
	maxIntensity = engineMaxPwmValue();
	intensityStep = maxIntensity / intensityStepCnt;

	P1DIR |= BIT4 | BIT5;
	P1OUT &= ~(BIT4 | BIT5);
}

void hmiExec() {
	if (intensitySwitch.get()) {
		intensity += intensityStep;
		if (intensity > maxIntensity) {
			intensity = maxIntensity;
		}
		debugWrite(intensity);
		engineSetPwmValue(intensity);
	}

	if (onTimeSwitch.get()) {
		engineIncOnTime();
	}

	if (offTimeSwitch.get()) {
		engineIncOffTime();
	}
}
